@@ -5,6 +5,7 @@ import functools
 import inspect
 import os
 import re
+import shutil
 import sys
 import types
 import typing as tp
@@ -218,9 +219,7 @@ class Aliases(cabc.MutableMapping):
         return str(self._raw)
 
     def __repr__(self):
-        return "{}.{}({})".format(
-            self.__class__.__module__, self.__class__.__name__, self._raw
-        )
+        return f"{self.__class__.__module__}.{self.__class__.__name__}({self._raw})"
 
     _repr_pretty_ = to_repr_pretty_
 
@@ -255,6 +254,8 @@ class ExecAlias:
                 locs=frame.f_locals,
                 filename=self.filename,
             )
+        if XSH.history is not None:
+            return XSH.history.last_cmd_rtn
 
     def __repr__(self):
         return f"ExecAlias({self.src!r}, filename={self.filename!r})"
@@ -284,9 +285,7 @@ class PartialEvalAliasBase:
         return self.f(args, stdin, stdout, stderr, spec, stack)
 
     def __repr__(self):
-        return "{name}({f!r}, acc_args={acc_args!r})".format(
-            name=self.__class__.__name__, f=self.f, acc_args=self.acc_args
-        )
+        return f"{self.__class__.__name__}({self.f!r}, acc_args={self.acc_args!r})"
 
 
 class PartialEvalAlias0(PartialEvalAliasBase):
@@ -406,7 +405,7 @@ def xonsh_reset(args, stdin=None):
 
 def source_foreign_fn(
     shell: str,
-    files_or_code: Annotated[tp.List[str], Arg(nargs="+")],
+    files_or_code: Annotated[list[str], Arg(nargs="+")],
     interactive=True,
     login=False,
     envcmd=None,
@@ -478,7 +477,7 @@ def source_foreign_fn(
         if not suppress_skip_message
         else suppress_skip_message
     )
-    files: tp.Tuple[str, ...] = ()
+    files: tuple[str, ...] = ()
     if prevcmd:
         pass  # don't change prevcmd if given explicitly
     elif os.path.isfile(files_or_code[0]):
@@ -541,7 +540,7 @@ def source_foreign_fn(
             msg = (
                 "Skipping application of {0!r} alias from {1!r} "
                 "since it shares a name with an existing xonsh alias. "
-                'Use "--overwrite-alias" option to apply it anyway.'
+                'Use "--overwrite-alias" option to apply it anyway. '
                 'You may prevent this message with "--suppress-skip-message" or '
                 '"$FOREIGN_ALIASES_SUPPRESS_SKIP_MESSAGE = True".'
             )
@@ -604,7 +603,7 @@ def source_alias(args, stdin=None):
 
 
 def source_cmd_fn(
-    files: Annotated[tp.List[str], Arg(nargs="+")],
+    files: Annotated[list[str], Arg(nargs="+")],
     login=False,
     aliascmd=None,
     extra_args="",
@@ -687,7 +686,7 @@ source_cmd = ArgParserAlias(func=source_cmd_fn, has_args=True, prog="source-cmd"
 
 
 def xexec_fn(
-    command: Annotated[tp.List[str], Arg(nargs=argparse.REMAINDER)],
+    command: Annotated[list[str], Arg(nargs=argparse.REMAINDER)],
     login=False,
     clean=False,
     name="",
@@ -809,11 +808,21 @@ def detect_xpip_alias():
 
     basecmd = [sys.executable, "-m", "pip"]
     try:
-        if ON_WINDOWS or IN_APPIMAGE:
+        if ON_WINDOWS:
             # XXX: Does windows have an installation mode that requires UAC?
             return basecmd
+        elif IN_APPIMAGE:
+            # In AppImage `sys.executable` is equal to path to xonsh.AppImage file and the real python executable is in $_
+            return [
+                XSH.env.get("_", "APPIMAGE_PYTHON_EXECUTABLE_NOT_FOUND"),
+                "-m",
+                "pip",
+            ]
         elif not os.access(os.path.dirname(sys.executable), os.W_OK):
-            return ["sudo"] + basecmd
+            return (
+                sys.executable
+                + " -m pip @(['install', '--user'] + $args[1:] if $args and $args[0] == 'install' else $args)"
+            )
         else:
             return basecmd
     except Exception:
@@ -893,7 +902,8 @@ def make_default_aliases():
             # Add aliases specific to the Anaconda python distribution.
             default_aliases["activate"] = ["source-cmd", "activate.bat"]
             default_aliases["deactivate"] = ["source-cmd", "deactivate.bat"]
-        if not locate_binary("sudo"):
+        if shutil.which("sudo", path=XSH.env.get_detyped("PATH")):
+            # XSH.commands_cache is not available during setup
             import xonsh.winutils as winutils
 
             def sudo(args):
